"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeviceMessageHandler = void 0;
const error_1 = require("../error");
const command_1 = require("./command");
class DeviceMessageHandler {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    static async handle(message, driver, client) {
        const { serialNumber, command } = message;
        const device = driver.getDevice(serialNumber);
        const station = driver.getStation(device.getStationSerial());
        switch (command) {
            case command_1.DeviceCommand.setStatusLed:
                await station.setStatusLed(device, message.value);
                return {};
            case command_1.DeviceCommand.setAutoNightVision:
                await station.setAutoNightVision(device, message.value);
                return {};
            case command_1.DeviceCommand.setMotionDetection:
                await station.setMotionDetection(device, message.value);
                return {};
            case command_1.DeviceCommand.setSoundDetection:
                await station.setSoundDetection(device, message.value);
                return {};
            case command_1.DeviceCommand.setPetDetection:
                await station.setPetDetection(device, message.value);
                return {};
            case command_1.DeviceCommand.setRTSPStream:
                await station.setRTSPStream(device, message.value);
                return {};
            case command_1.DeviceCommand.setAntiTheftDetection:
                await station.setAntiTheftDetection(device, message.value);
                return {};
            case command_1.DeviceCommand.setWatermark:
                await station.setWatermark(device, message.value);
                return {};
            case command_1.DeviceCommand.enableDevice:
                await station.enableDevice(device, message.value);
                return {};
            case command_1.DeviceCommand.lockDevice:
                await station.lockDevice(device, message.value);
                return {};
            case command_1.DeviceCommand.getPropertiesMetadata:
                {
                    const properties = device.getPropertiesMetadata();
                    return { properties: properties };
                }
            case command_1.DeviceCommand.getProperties:
                {
                    const properties = device.getProperties();
                    return { properties: properties };
                }
            case command_1.DeviceCommand.setProperty:
                driver.setDeviceProperty(serialNumber, message.name, message.value);
                return {};
            default:
                throw new error_1.UnknownCommandError(command);
        }
    }
}
exports.DeviceMessageHandler = DeviceMessageHandler;
//# sourceMappingURL=message_handler.js.map