"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DriverMessageHandler = void 0;
const error_1 = require("../error");
const command_1 = require("./command");
class DriverMessageHandler {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    static async handle(message, driver, client) {
        const { command } = message;
        switch (command) {
            case command_1.DriverCommand.setVerifyCode:
                {
                    const result = await driver.connect(message.verifyCode);
                    return { result: result };
                }
            case command_1.DriverCommand.pollRefresh:
                await driver.refreshData();
                return {};
            case command_1.DriverCommand.isConnected:
                {
                    const result = await driver.isConnected();
                    return { connected: result };
                }
            case command_1.DriverCommand.isPushConnected:
                {
                    const result = await driver.isPushConnected();
                    return { connected: result };
                }
            case command_1.DriverCommand.connect:
                {
                    const result = await driver.connect();
                    return { connected: result };
                }
            case command_1.DriverCommand.disconnect:
                await driver.close();
                return {};
            default:
                throw new error_1.UnknownCommandError(command);
        }
    }
}
exports.DriverMessageHandler = DriverMessageHandler;
//# sourceMappingURL=message_handler.js.map